/*	$NetBSD: logwtmp.c,v 1.5 1997/06/18 19:05:51 christos Exp $	*/

/*
 * Copyright (c) 1988, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */


#include <sys/cdefs.h>
#ifndef lint
#if 0
static char sccsid[] = "@(#)logwtmp.c	8.1 (Berkeley) 6/4/93";
#else
__RCSID("$NetBSD: logwtmp.c,v 1.5 1997/06/18 19:05:51 christos Exp $");
#endif
#endif /* not lint */

#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>

#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include "extern.h"

static int fd = -1;

/* <sjg> much of this from my portable login(1) */

#define ARRSTR(a, s) \
	(strncpy((a), s, sizeof (a)))

#if defined(__svr4__) || defined(HAVE_UTMPX_H)
# define UTMPX
# ifdef sun
#   define HAVE_GETUTXID
# endif
#endif

#ifdef __hpux__
/*
 * HP-UX 10 has UTMPX but does not use it.
 * Just pretend we are still 9.
 */
# undef UTMPX
#endif
	  
#ifdef UTMPX
# include <utmpx.h>
# ifndef ut_name
#   define ut_name ut_user
# endif
# ifndef ut_xtime
#   define ut_xtime ut_tv.tv_sec
# endif
# define UTMP_T struct utmpx
# define ut_time ut_xtime
# define utmp utmpx
# define HAVE_UT_HOST
# ifdef HAVE_GETUTXID
# define getutent getutxent
# define getutid getutxid
# define getutline getutxline
# define pututline pututxline
# define setutent setutxent
# define endutent endutxent
# endif
# ifdef WTMPX_FILE
#   undef _PATH_WTMP
#   define _PATH_WTMP WTMPX_FILE
# endif
#else
# include <utmp.h>
# define UTMP_T struct utmp
#endif
#include "pathnames.h"

static UTMP_T ut, *utp = 0;

#ifndef SC_WILDC
# define SC_WILDC 0xff
#endif

/*
 * Modified version of logwtmp that holds wtmp file open
 * after first call, for use with ftp (which may chroot
 * after login, but before logout).
 */
void
logwtmp(line, name, host)
	char *line, *name, *host;
{
	UTMP_T ut;
	struct stat buf;
	int i;

	if (fd < 0 && (fd = open(_PATH_WTMP, O_WRONLY|O_APPEND, 0)) < 0)
		return;
	if (fstat(fd, &buf) == 0) {
		memset((char *)&ut, 0, sizeof(ut));
		ARRSTR(ut.ut_line, line);
#if defined(HAVE_UT_ID) || defined(HAVE_GETUTID) || defined(HAVE_GETXUTID) || defined(__hpux__) || defined(__svr4__) 
		ut.ut_pid = getpid();
		if ((i = strlen(line)) > sizeof(ut.ut_id)) {
			/*
			 * <sjg> the last few bytes will vary more 
			 * than the first few
			 */
			(void)strncpy(ut.ut_id, &line[i - sizeof(ut.ut_id)],
			      sizeof(ut.ut_id));
			ut.ut_id[0] = 'f';
		} else {
			ARRSTR(ut.ut_id, line);
		}
#ifdef USER_PROCESS
		ut.ut_type = (name && *name) ? USER_PROCESS : DEAD_PROCESS;
#endif
#endif
		ARRSTR(ut.ut_name, name);
#if defined(BSD) || defined(sun) || defined(HAVE_UT_HOST)
		if (host)
			ARRSTR(ut.ut_host, host);
#endif
		(void)time(&ut.ut_time);
#if defined(HAVE_MAKEUTX)
		makeutx(&ut);
#endif
		if (write(fd, (char *)&ut, sizeof(ut)) !=
		    sizeof(ut))
			(void)ftruncate(fd, buf.st_size);
	}
}
